use eframe::egui::{self, Response, RichText, Ui, Widget};
use std::sync::mpsc::{Receiver, Sender};

use crate::groq::{types::ChatCompletionResponse, GROQ_CLIENT};

pub struct ModelResponse {
    pub name: String,
    message: String,
    pub tokens: i32,
    pub time: f32,
    loading: bool,
    tx: Sender<ChatCompletionResponse>,
    rx: Receiver<ChatCompletionResponse>,
}

impl ModelResponse {
    pub fn new(name: String, message: String, tokens: i32, time: f32) -> Self {
        let (tx, rx) = std::sync::mpsc::channel();

        Self {
            name,
            message,
            tokens,
            time,
            loading: false,
            tx,
            rx,
        }
    }

    pub fn start_chat_completion(&mut self, prompt: String) {
        self.message = "Loading...".to_string();
        self.tokens = 0;
        self.time = 0.0;
        self.loading = true;
        Self::spawn_chat_completion_task(prompt, self.name.clone(), self.tx.clone());
    }

    fn check_completion(&mut self) {
        if let Ok(response) = self.rx.try_recv() {
            self.message = response.choices[0].message.content.clone();
            self.tokens = response.usage.total_tokens;
            self.time = response.usage.total_time;
            self.loading = false;
        }
    }

    fn spawn_chat_completion_task(
        prompt: String,
        model_name: String,
        tx: Sender<ChatCompletionResponse>,
    ) {
        tokio::spawn(async move {
            let response = GROQ_CLIENT.chat_completion(model_name, prompt).await;
            tx.send(response).expect("Failed to send response");
        });
    }
}

impl Widget for &mut ModelResponse {
    fn ui(self, ui: &mut Ui) -> Response {
        if self.loading {
            self.check_completion();
        }

        ui.vertical(|ui| {
            egui::Frame::none()
                .inner_margin(8.0)
                .rounding(4.0)
                .fill(egui::Color32::DARK_GRAY)
                .show(ui, |ui| {
                    ui.label(RichText::new(&self.name).strong().heading());

                    ui.horizontal_wrapped(|ui| {
                        ui.horizontal(|ui| {
                            ui.label("Tokens:");
                            ui.label(self.tokens.to_string());
                        });

                        ui.horizontal(|ui| {
                            ui.label("Time:");
                            ui.label(self.time.to_string());
                        });
                    });

                    egui::Frame::none()
                        .inner_margin(8.0)
                        .rounding(4.0)
                        .fill(egui::Color32::WHITE)
                        .show(ui, |ui| {
                            ui.label(
                                egui::RichText::new(&self.message).color(egui::Color32::BLACK),
                            );
                        });
                })
        })
        .response
    }
}
